@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# pylint: disable=ungrouped-imports
 """Defined constants."""
 
 import datetime
@@ -12,16 +13,25 @@ import re
 import shutil
 import sys
 import threading
+from typing import TYPE_CHECKING
 
 import peewee
 import playhouse.db_url
 import redis
 import stem.util.term
 
-import darc.typing as typing
 from darc._compat import nullcontext
 from darc.error import render_error
 
+if TYPE_CHECKING:
+    from datetime import timedelta
+    from multiprocessing import Lock as ProcessLock
+    from threading import Lock as ThreadLock
+    from typing import Optional, Union
+
+    from peewee import Database
+    from redis import Redis
+
 # reboot mode?
 REBOOT = bool(int(os.getenv('DARC_REBOOT', '0')))
 
@@ -43,9 +53,12 @@ ROOT = os.path.dirname(os.path.abspath(__file__))
 CWD = os.path.realpath(os.curdir)
 
 # process number
-DARC_CPU = os.getenv('DARC_CPU')
-if DARC_CPU is not None:
-    DARC_CPU = int(DARC_CPU)
+_DARC_CPU = os.getenv('DARC_CPU')
+if _DARC_CPU is not None:
+    DARC_CPU = int(_DARC_CPU)
+else:
+    DARC_CPU = 1
+del _DARC_CPU
 
 # use multiprocessing?
 FLAG_MP = bool(int(os.getenv('DARC_MULTIPROCESSING', '1')))
@@ -101,6 +114,7 @@ if DEBUG:
     print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
                                 stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
 MIME_WHITE_LIST = [re.compile(mime, re.IGNORECASE) for mime in _MIME_WHITE_LIST]
+del _MIME_WHITE_LIST
 
 # content type black list
 _MIME_BLACK_LIST = json.loads(os.getenv('MIME_BLACK_LIST', '[]'))
@@ -111,6 +125,7 @@ if DEBUG:
     print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
                                 stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
 MIME_BLACK_LIST = [re.compile(mime, re.IGNORECASE) for mime in _MIME_BLACK_LIST]
+del _MIME_BLACK_LIST
 
 # content type fallback value
 MIME_FALLBACK = bool(int(os.getenv('MIME_FALLBACK', '0')))
@@ -124,6 +139,7 @@ if DEBUG:
     print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
                                 stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
 PROXY_BLACK_LIST = [proxy.casefold() for proxy in _PROXY_BLACK_LIST]
+del _PROXY_BLACK_LIST
 
 # proxy type white list
 _PROXY_WHITE_LIST = json.loads(os.getenv('PROXY_WHITE_LIST', '[]').casefold())
@@ -134,6 +150,7 @@ if DEBUG:
     print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
                                 stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
 PROXY_WHITE_LIST = [proxy.casefold() for proxy in _PROXY_WHITE_LIST]
+del _PROXY_WHITE_LIST
 
 # proxy type fallback value
 PROXY_FALLBACK = bool(int(os.getenv('PROXY_FALLBACK', '0')))
@@ -141,7 +158,7 @@ PROXY_FALLBACK = bool(int(os.getenv('PROXY_FALLBACK', '0')))
 # time delta for caches in seconds
 _TIME_CACHE = float(os.getenv('TIME_CACHE', '60'))
 if math.isfinite(_TIME_CACHE):
-    TIME_CACHE = datetime.timedelta(seconds=_TIME_CACHE)
+    TIME_CACHE = datetime.timedelta(seconds=_TIME_CACHE)  # type: Optional[timedelta]
 else:
     TIME_CACHE = None
 del _TIME_CACHE
@@ -149,7 +166,7 @@ del _TIME_CACHE
 # selenium wait time
 _SE_WAIT = float(os.getenv('SE_WAIT', '60'))
 if math.isfinite(_SE_WAIT):
-    SE_WAIT = _SE_WAIT
+    SE_WAIT = _SE_WAIT  # type: Optional[float]
 else:
     SE_WAIT = None
 del _SE_WAIT
@@ -160,7 +177,7 @@ SE_EMPTY = '<html><head></head><body></body></html>'
 # selenium wait time
 _DARC_WAIT = float(os.getenv('DARC_WAIT', '60'))
 if math.isfinite(_DARC_WAIT):
-    DARC_WAIT = _DARC_WAIT
+    DARC_WAIT = _DARC_WAIT  # type: Optional[float]
 else:
     DARC_WAIT = None
 del _DARC_WAIT
@@ -168,11 +185,12 @@ del _DARC_WAIT
 # Redis client
 _REDIS_URL = os.getenv('REDIS_URL')
 if _REDIS_URL is None:
-    REDIS = NotImplemented
+    REDIS = None  # type: Optional[Redis]
     FLAG_DB = True
 else:
     REDIS = redis.Redis.from_url(_REDIS_URL, decode_components=True)
     FLAG_DB = False
+del _REDIS_URL
 
 # database instance
 _DB_URL = os.getenv('DB_URL')
@@ -180,8 +198,8 @@ if _DB_URL is None:
     DB = peewee.SqliteDatabase(f'sqlite://{PATH_DB}/sqlite/darc.db')
     DB_WEB = peewee.SqliteDatabase(f'sqlite://{PATH_DB}/sqlite/darcweb.db')
 else:
-    DB: typing.Database = playhouse.db_url.connect(f'{_DB_URL}/darc', unquote_password=True)
-    DB_WEB: typing.Database = playhouse.db_url.connect(f'{_DB_URL}/darcweb', unquote_password=True)
+    DB = playhouse.db_url.connect(f'{_DB_URL}/darc', unquote_password=True)  # type: Database # type: ignore[no-redef]
+    DB_WEB = playhouse.db_url.connect(f'{_DB_URL}/darcweb', unquote_password=True)  # type: Database # type: ignore[no-redef] # pylint: disable=line-too-long
 del _DB_URL
 
 
@@ -205,7 +223,7 @@ def getpid() -> int:
     return -1
 
 
-def get_lock() -> typing.Union[multiprocessing.Lock, threading.Lock, nullcontext]:
+def get_lock() -> 'Union[ProcessLock, ThreadLock, nullcontext]':  # type: ignore[valid-type]
     """Get a lock.
 
     Returns:
