@@ -13,13 +13,21 @@ representing URLs, specifically from ``requests`` and
 
 """
 
-import peewee
+from typing import TYPE_CHECKING
+
+from peewee import BooleanField, CharField, DateTimeField, ForeignKeyField, TextField
 
-import darc.typing as typing
 from darc.model.abc import BaseModelWeb as BaseModel
 from darc.model.utils import IntEnumField, Proxy
 from darc.model.web.hostname import HostnameModel
 
+if TYPE_CHECKING:
+    from typing import List
+
+    from darc._compat import datetime
+    from darc.model.web.requests import RequestsModel
+    from darc.model.web.selenium import SeleniumModel
+
 __all__ = ['URLModel']
 
 
@@ -33,22 +41,30 @@ class URLModel(BaseModel):
 
     """
 
+    #: ``requests`` submission record, back reference from
+    #: :attr:`RequestsModel.url <darc.models.web.requests.RequestsModel.url>`.
+    requests: 'List[RequestsModel]'
+
+    #: ``selenium`` submission record, back reference from
+    #: :attr:`SeleniumModel.url <darc.models.web.selenium.SeleniumModel.url>`.
+    selenium: 'List[SeleniumModel]'
+
     #: Original URL (c.f. :attr:`link.url <darc.link.Link.url>`).
-    url: str = peewee.TextField()
+    url: str = TextField()
     #: Sha256 hash value (c.f. :attr:`Link.name <darc.link.Link.name>`).
-    hash: str = peewee.CharField(max_length=256, unique=True)
+    hash: str = CharField(max_length=256, unique=True)
 
     #: Hostname (c.f. :attr:`link.host <darc.link.Link.host>`).
-    hostname: HostnameModel = peewee.ForeignKeyField(HostnameModel, backref='urls')
+    hostname: 'HostnameModel' = ForeignKeyField(HostnameModel, backref='urls')
     #: Proxy type (c.f. :attr:`link.proxy <darc.link.Link.proxy>`).
-    proxy: Proxy = IntEnumField(choices=Proxy)
+    proxy: 'Proxy' = IntEnumField(choices=Proxy)
 
     #: Timestamp of first submission.
-    discovery: typing.Datetime = peewee.DateTimeField()
+    discovery: 'datetime' = DateTimeField()
     #: Timestamp of last submission.
-    last_seen: typing.Datetime = peewee.DateTimeField()
+    last_seen: 'datetime' = DateTimeField()
 
     #: If the hostname is still active.
-    alive: bool = peewee.BooleanField()
+    alive: bool = BooleanField()
     #: The hostname is active/inactive since this timestamp.
-    since: typing.Datetime = peewee.DateTimeField()
+    since: 'datetime' = DateTimeField()
