@@ -12,15 +12,18 @@ without further processing.
 """
 
 import os
+from typing import TYPE_CHECKING
 
 from darc.const import PATH_MISC, get_lock
-from darc.link import Link
+
+if TYPE_CHECKING:
+    from darc.link import Link
 
 PATH = os.path.join(PATH_MISC, 'script.txt')
 LOCK = get_lock()
 
 
-def save_script(link: Link) -> None:
+def save_script(link: 'Link') -> None:
     """Save JavaScript link.
 
     The function will save JavaScript link to the file
@@ -30,6 +33,6 @@ def save_script(link: Link) -> None:
         link: Link object representing the JavaScript link.
 
     """
-    with LOCK:  # type: ignore
+    with LOCK:  # type: ignore[union-attr]
         with open(PATH, 'a') as file:
             print(link.url, file=file)
