@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# pylint: disable=ungrouped-imports
 """Requests Wrapper
 ======================
 
@@ -8,18 +9,26 @@ project.
 
 """
 
+from typing import TYPE_CHECKING
+
 import requests
 import requests_futures.sessions
 
-import darc.typing as typing
 from darc.const import DARC_CPU
 from darc.error import UnsupportedLink
-from darc.link import Link
 from darc.proxy.i2p import I2P_REQUESTS_PROXY
 from darc.proxy.tor import TOR_REQUESTS_PROXY
 
+if TYPE_CHECKING:
+    from typing import Optional, Union
 
-def default_user_agent(name: str = 'python-darc', proxy: typing.Optional[str] = None) -> str:
+    from requests import Session
+    from requests_futures.sessions import FuturesSession
+
+    from darc.link import Link
+
+
+def default_user_agent(name: str = 'python-darc', proxy: 'Optional[str]' = None) -> str:
     """Generates the default user agent.
 
     Args:
@@ -39,12 +48,12 @@ def default_user_agent(name: str = 'python-darc', proxy: typing.Optional[str] =
     return ua
 
 
-def request_session(link: Link, futures: bool = False) -> typing.Union[typing.Session, typing.FuturesSession]:
+def request_session(link: 'Link', futures: bool = False) -> 'Union[Session, FuturesSession]':
     """Get requests session.
 
     Args:
         link: Link requesting for requests.Session.
-        futures: If returns a requests_futures.FuturesSession.
+        futures: If returns a :class:`requests_futures.FuturesSession`.
 
     Returns:
         Union[requests.Session, requests_futures.FuturesSession]:
@@ -64,14 +73,14 @@ def request_session(link: Link, futures: bool = False) -> typing.Union[typing.Se
     if factory is None:
         raise UnsupportedLink(link.url)
 
-    return factory(futures=futures)
+    return factory(futures=futures)  # type: ignore
 
 
-def i2p_session(futures: bool = False) -> typing.Union[typing.Session, typing.FuturesSession]:
+def i2p_session(futures: bool = False) -> 'Union[Session, FuturesSession]':
     """I2P (.i2p) session.
 
     Args:
-        futures: If returns a requests_futures.FuturesSession.
+        futures: If returns a :class:`requests_futures.FuturesSession`.
 
     Returns:
         Union[requests.Session, requests_futures.FuturesSession]:
@@ -91,11 +100,11 @@ def i2p_session(futures: bool = False) -> typing.Union[typing.Session, typing.Fu
     return session
 
 
-def tor_session(futures: bool = False) -> typing.Union[typing.Session, typing.FuturesSession]:
+def tor_session(futures: bool = False) -> 'Union[Session, FuturesSession]':
     """Tor (.onion) session.
 
     Args:
-        futures: If returns a requests_futures.FuturesSession.
+        futures: If returns a :class:`requests_futures.FuturesSession`.
 
     Returns:
         Union[requests.Session, requests_futures.FuturesSession]:
@@ -115,11 +124,11 @@ def tor_session(futures: bool = False) -> typing.Union[typing.Session, typing.Fu
     return session
 
 
-def null_session(futures: bool = False) -> typing.Union[typing.Session, typing.FuturesSession]:
+def null_session(futures: bool = False) -> 'Union[Session, FuturesSession]':
     """No proxy session.
 
     Args:
-        futures: If returns a requests_futures.FuturesSession.
+        futures: If returns a :class:`requests_futures.FuturesSession`.
 
     Returns:
         Union[requests.Session, requests_futures.FuturesSession]:
