@@ -7,9 +7,13 @@ import contextlib
 import datetime
 import os
 import shutil
-import subprocess  # nosec
+import subprocess  # nosec: B404
 import sys
 import time
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from argparse import ArgumentParser
 
 # root path
 ROOT = os.path.dirname(os.path.abspath(__file__))
@@ -19,7 +23,7 @@ SCPT = os.path.join(ROOT, 'upload.sh')
 ONEDAY = datetime.timedelta(days=1)
 
 
-def upload(path, host, user):
+def upload(path: str, host: str, user: str) -> None:
     """Upload files."""
     today = datetime.date.today()
     today_str = today.isoformat()
@@ -36,15 +40,15 @@ def upload(path, host, user):
     print(f'[{datetime.datetime.now().isoformat()}] Archiving & uploading API submission files...')
     with open('logs/upload.log', 'at', buffering=1) as log_file:
         with contextlib.suppress(subprocess.CalledProcessError):
-            subprocess.check_call(['bash', SCPT], env=os.environ.update(dict(  # nosec
-                HOST=host,
-                USER=user,
-                DATE=yesterday,
-            )), cwd=path, stdout=log_file, stderr=subprocess.STDOUT)
+            subprocess.check_call(['bash', SCPT], env=os.environ.update({  # nosec
+                'HOST': host,
+                'USER': user,
+                'DATE': yesterday,
+            }), cwd=path, stdout=log_file, stderr=subprocess.STDOUT)
     print(f'[{datetime.datetime.now().isoformat()}] Uploaded API submission files...')
 
 
-def get_parser():
+def get_parser() -> 'ArgumentParser':
     """Argument parser."""
     parser = argparse.ArgumentParser('upload',
                                      description='upload API submission files')
@@ -56,7 +60,7 @@ def get_parser():
     return parser
 
 
-def main():
+def main() -> int:
     """Entryprocess."""
     parser = get_parser()
     args = parser.parse_args()
